
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Iterator;

public class Simulator implements Iterable<SimObject> {

    private final int PERIOD = 100; //ms
    private boolean running = false;
    private volatile boolean quit = false;
    private volatile boolean changed = false;
    ArrayList<SimObject> objects;
    World world;

    public Simulator() {
        objects = new ArrayList();
        world = new World(new Dimension(640, 480));
    }

    public World getWorld() {
        return world;
    }

    @Override
    public Iterator iterator() {
        return objects.iterator();
    }

    public void addObject(SimObject obj) {
        objects.add(obj);
    }

    public synchronized void start() {
        quit = false;
        new Thread(this::doSimulation).start();
    }

    public boolean isRunning() {
        return running;
    }

    public synchronized void shutdown() {
        quit = true;
    }

    public synchronized boolean isChanged() {
        boolean ret = changed;
        changed = false;
        return ret;
    }

    private void doSimulation() {
        running = true;
        boolean q = false;
        while (!q) {
            long t1 = System.currentTimeMillis();
            for (SimObject obj : objects) {
                obj.doStep();
            }
            long t2 = System.currentTimeMillis();
            try {
                // What about faster simulation step 
                // than 1 ms? use System.nanoTime();
                Thread.sleep(PERIOD - (t2 - t1));
            } catch (InterruptedException e) {
            }
            synchronized (this) {
                q = quit;
                changed = true;
            }
        }
        running = false;
    }
}
